@@ -5,7 +5,7 @@
 /*  This file contains routines to parse and evaluate          */
 /*  expressions.                                               */
 /*                                                             */
-/*  Copyright 1992-2020 by Dianne Skoll                        */
+/*  Copyright 1992-2021 by Dianne Skoll                        */
 /*                                                             */
 /***************************************************************/
 
@@ -13,6 +13,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <limits.h>
 
 #include <stdlib.h>
 
@@ -493,6 +494,7 @@ static int MakeValue(char const *s, Value *v, Var *locals, ParsePtr p)
     int len;
     int h, m, r;
     int ampm = 0;
+    int prev_val;
 
     if (*s == '\"') { /* It's a literal string "*/
 	len = strlen(s)-1;
@@ -519,9 +521,15 @@ static int MakeValue(char const *s, Value *v, Var *locals, ParsePtr p)
 	return OK;
     } else if (isdigit(*s)) { /* It's a number - use len to hold it.*/
 	len = 0;
+        prev_val = 0;
 	while (*s && isdigit(*s)) {
 	    len *= 10;
 	    len += (*s++ - '0');
+            if (len < prev_val) {
+                /* We overflowed */
+                return E_2HIGH;
+            }
+            prev_val = len;
 	}
 	if (*s == ':' || *s == '.' || *s == TimeSep) { /* Must be a literal time */
 	    s++;
@@ -758,15 +766,22 @@ static int Add(void)
 
 /* If both are ints, just add 'em */
     if (v2.type == INT_TYPE && v1.type == INT_TYPE) {
-	v2.v.val += v1.v.val;
-	PushValStack(v2);
+        int old = v1.v.val;
+	v1.v.val += v2.v.val;
+        /* Check for overflow */
+        if (_private_add_overflow(v1.v.val, v2.v.val, old)) {
+            return E_2HIGH;
+        }
+	PushValStack(v1);
 	return OK;
     }
 
 /* If it's a date plus an int, add 'em */
     if ((v1.type == DATE_TYPE && v2.type == INT_TYPE) ||
 	(v1.type == INT_TYPE && v2.type == DATE_TYPE)) {
+        int old = v1.v.val;
 	v1.v.val += v2.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	v1.type = DATE_TYPE;
 	PushValStack(v1);
@@ -776,7 +791,9 @@ static int Add(void)
 /* If it's a datetime plus an int or a time, add 'em */
     if ((v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) ||
 	((v1.type == INT_TYPE || v1.type == TIME_TYPE) && v2.type == DATETIME_TYPE)) {
+        int old = v1.v.val;
 	v1.v.val += v2.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	v1.type = DATETIME_TYPE;
 	PushValStack(v1);
@@ -788,7 +805,10 @@ static int Add(void)
     if ((v1.type == TIME_TYPE && v2.type == INT_TYPE) ||
 	(v1.type == INT_TYPE && v2.type == TIME_TYPE) ||
 	(v1.type == TIME_TYPE && v2.type == TIME_TYPE)) {
-	v1.v.val = (v1.v.val + v2.v.val) % MINUTES_PER_DAY;
+        int old = v1.v.val;
+	v1.v.val += v2.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
+	v1.v.val = v1.v.val % MINUTES_PER_DAY;
 	if (v1.v.val < 0) v1.v.val += MINUTES_PER_DAY;
 	v1.type = TIME_TYPE;
 	PushValStack(v1);
@@ -848,14 +868,18 @@ static int Subtract(void)
 
     /* If they're both INTs, do subtraction */
     if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
+        int old = v1.v.val;
 	v1.v.val -= v2.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_2HIGH;
 	PushValStack(v1);
 	return OK;
     }
 
     /* If it's a date minus an int, do subtraction, checking for underflow */
     if (v1.type == DATE_TYPE && v2.type == INT_TYPE) {
+        int old = v1.v.val;
 	v1.v.val -= v2.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	PushValStack(v1);
 	return OK;
@@ -864,7 +888,9 @@ static int Subtract(void)
     /* If it's a datetime minus an int or a time, do subtraction,
      * checking for underflow */
     if (v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) {
+        int old = v1.v.val;
 	v1.v.val -= v2.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	PushValStack(v1);
 	return OK;
@@ -882,7 +908,9 @@ static int Subtract(void)
     if ((v1.type == TIME_TYPE && v2.type == TIME_TYPE) ||
 	(v1.type == DATETIME_TYPE && v2.type == DATETIME_TYPE) ||
 	(v1.type == DATE_TYPE && v2.type == DATE_TYPE)) {
+        int old = v1.v.val;
 	v1.v.val -= v2.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	v1.type = INT_TYPE;
 	PushValStack(v1);
 	return OK;
@@ -912,7 +940,11 @@ static int Multiply(void)
     }
 
     if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
+        int old = v1.v.val;
 	v1.v.val *= v2.v.val;
+        if (v2.v.val != 0) {
+            if (_private_div(v1.v.val, v2.v.val) != old) return E_2HIGH;
+        }
 	PushValStack(v1);
 	return OK;
     }
@@ -940,6 +972,10 @@ static int Divide(void)
 
     if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
 	if (v2.v.val == 0) return E_DIV_ZERO;
+        /* This is the only way it can overflow */
+        if (v2.v.val == -1 && v1.v.val == INT_MIN) {
+            return E_2HIGH;
+        }
 	v1.v.val /= v2.v.val;
 	PushValStack(v1);
 	return OK;
@@ -1114,7 +1150,9 @@ static int UnMinus(void)
 {
     Value *v = &ValStack[ValStackPtr-1];
     if (v->type != INT_TYPE) return E_BAD_TYPE;
+    int old = v->v.val;
     v->v.val = -v->v.val;
+    if (_private_unminus_overflow(old, v->v.val)) return E_2HIGH;
     return OK;
 }
 
